/*----------------------------------------------------------------------------
 Copyright:
 Author:         Radig Ulrich
 Remarks:
 known Problems: none
 Version:        21.11.2009
 Description:    Description:    EA DOG (M/L)128-6
------------------------------------------------------------------------------*/
/*
Ausgangspunkt dieser Datei ist der Programmcode siehe oben.
Es sind einige Vernderung/Vereinfachungen vorgenommen:

- Beschrnkung auf den Normal-Font mit den ASCII-Zeichen 32 ... 127
  (das spart Speicherplatz im eeprom, der dann genutzt wird fr:)
- Zustzlich der doppelte breite/hohe Big-Font mit den ASCII-Zeichen 32...127
- Auf die Skalierbarkeit von Fonts wurde verzichtet.
- In den Funktionen DOG_write_data() und DOG_write_cmd() wird A0 nur verndert,
  wenn es wirklich erforderlich ist.
- Zustzliche sind entstanden:
  Bargraph(),
  Dezimalpunkt mit halber Chr-Breite
  Led-Helligkeit und Kontrast einstellen
*/

#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <ctype.h>
#include <avr/io.h>
#include "global.h"
#include "dog128x64.h"
#include "dog_font.h"
#include "dog_gfx.h"

// --- lokale Funktionen
void DOG_write_data(uint8_t data);
void DOG_write_cmd(uint8_t cmd);
void DOG_write_xy(void);

#if DEMO == BARGRAPH_DEMO
#define  H_BARGRAPH
#endif

// --- lokale Variablen
uint8_t txt_cursor_x, txt_cursor_y;		// aktuelle Cursorposition
uint8_t font_typ;

//uint8_t lcd_contrast = 0x16;			// schwarz
uint8_t lcd_contrast = 0x23;				// gelb-grn

// ----------------------------------------------------------------------------
// Initialisiert:
// - Timer2 fr DOG LED-PWM
// - Hardware-SPI
// - Initialisiert DOG

void DOG_init()
{
// Der Timer2 fr die LED wird in DOG_dimm_led() konfiguriert

SPI_DDR   = (1<<SPI_Clock) | (1<<SPI_DO) | (1<<SPI_SS) | (1<<SPI_RESET) | (1<<SPI_A0);
SPI_PORT |= (1<<SPI_SS)   | (1<<SPI_A0) | (1<<SPI_RESET);
SPCR = (1<<SPE)|(1<<MSTR); 				// Enable SPI, SPI in Master Mode

SPI_PORT  = ~(1<<SPI_RESET);       		// Reset the Display Controller
asm volatile ("nop");
SPI_PORT |=  (1<<SPI_RESET);

SPSR = 1<<SPI2X;										// Takt Verdoppelung
SPCR = 1<<SPE | 1<<MSTR | 0<<SPR1 | 0<<SPR0; // 1/ CPU Takt * 2

DOG_write_cmd(DISPLAY_SCAN_DIR_REVERSE);		// C8
DOG_write_cmd(DISPLAY_BOTTOMVIEW);				// A0
DOG_write_cmd(DISPLAY_NORMAL);					// A6
DOG_write_cmd(DISPLAY_BIAS_1_9);					// A2
DOG_write_cmd(DISPLAY_POWER_ALL_ON);  			// 2F
DOG_write_cmd(DISPLAY_VOLTAGE_RESISTOR); 		// 26
DOG_write_cmd(DISPLAY_VOLUME_MODE_1);			// 81
DOG_write_cmd(lcd_contrast);						// Kontrast
DOG_write_cmd(DISPLAY_INDICATOR_OFF); 			// AC
DOG_write_cmd(DISPLAY_INDICATOR_MODE_OFF); 	// 00
DOG_write_cmd(DISPLAY_ON); 						// AF

//font_width = 6;
font_typ   = FONT_NORMAL;

DOG_cls();
}

// ----------------------------------------------------------------------------
// verndert den Kontrast
void DOG_lcd_contrast(int8_t i)
{
lcd_contrast += i;

DOG_write_cmd(DISPLAY_VOLUME_MODE_1);			// 81
DOG_write_cmd(lcd_contrast);						// Kontrast
}

// ----------------------------------------------------------------------------
// stellt die Helligkeit der LED-Hinterleuchtung ein
// wenn pwm = 0, dann wird der LED-Pin abgeschaltet (als Eingang geschaltet)
void DOG_dimm_led(uint8_t pwm)
{
if (pwm)
	{
	LED_DDR |= (1<<LED_PIN);					// PWM PORT auf Ausgang (OC2)
	TCCR2A = (1<<COM2B1 | 0<<WGM21 | 1<<WGM20); // PWM Phase correct, clear OC2B on compare match
//	TCCR2B = ( 1<<WGM22 | 1<<CS22 | 0<<CS21 | 0<<CS20 );		// 1/64 Takt -> ca. 113 Hz PWM-Takt
	TCCR2B = ( 1<<WGM22 | 0<<CS22 | 1<<CS21 | 1<<CS20 );		// 1/32 Takt -> ca. 226 Hz PWM-Takt
	OCR2B = pwm;
	OCR2A = 255;
	}

else
	{
	TCCR2B = 0;										// Timer2 abschalten
	LED_DDR  &= ~(1<<LED_PIN);					// LED-Pin als Eingang ohne Pullup
	LED_PORT &= ~(1<<LED_PIN);
	}
}

// ----------------------------------------------------------------------------
// Schreibt ein Datenbyte raus
// PIN_A0 wird dabei nicht verndert
void DOG_write_data(uint8_t data)
{
SPI_PORT &= ~(1<<SPI_SS);
SPDR = data;
while(!(SPSR & (1<<SPIF)));

SPI_PORT |= (1<<SPI_SS);
}

// ----------------------------------------------------------------------------
// Schreibt ein Commandbyte raus
// A0 muss zu Beginn gelscht und am Ende wieder gesetzt werden
void DOG_write_cmd(uint8_t cmd)
{
SPI_PORT &= ~((1<<SPI_SS) | (1<<PIN_A0));

SPDR = cmd;
while(!(SPSR & (1<<SPIF)));

SPI_PORT |=  ((1<<SPI_SS) | (1<<PIN_A0));
}

// ----------------------------------------------------------------------------
// Plaziert den Text-Cursor unter Bercksichtigung der Standard-Font-Breite (6),
// Zeile und Spalte zhlen jeweils ab 0 (arbeitet so, wie lcd_goto(x,y)).
// Schreibt die Cursor-Position in lokale Variablen
// txt_cursor_x/y wird bei der Zeichenausgabe zur Erkennung des Zeilenendes benutzt
//
// Der Ursprung 0/0 ist oben links,
// der Wertebereich spalte = [0..20], zeile = [0..7]

void DOG_txt_pos(uint8_t spalte, uint8_t zeile)
{
txt_cursor_x = spalte * FONT_NORMAL_WIDTH; 		// immer nach Standardfont zhlen
txt_cursor_y = zeile;

tmp = txt_cursor_x + DISPLAY_OFFSET;

DOG_write_cmd(DISPLAY_PAGE_ADDRESS    | ((zeile)    & 0x0F));
DOG_write_cmd(DISPLAY_COL_ADDRESS_MSB | ((tmp >>4)  & 0x0F));
DOG_write_cmd(DISPLAY_COL_ADDRESS_LSB | ((tmp)      & 0x0F));
}

// ----------------------------------------------------------------------------
// Erlaubt das feinere Plazieren des Cursors unabhngig von der Standard-Fontbreite,
// die X-Koordinate (Spalte) wird direkt angegeben,
// als Y-Koordinate ist nur die Zeilennummer von 0..7 mglich
//
// Der Ursprung 0/0 ist oben links,
// der Wertebereich x = [0..127], zeile = [0..7]
void DOG_goto_xy(uint8_t x, uint8_t zeile)
{
tmp = x + DISPLAY_OFFSET;

DOG_write_cmd(DISPLAY_PAGE_ADDRESS    | ((zeile)  & 0x0F));
DOG_write_cmd(DISPLAY_COL_ADDRESS_MSB | ((tmp>>4) & 0x0F));
DOG_write_cmd(DISPLAY_COL_ADDRESS_LSB | ((tmp)    & 0x0F));
}

// ----------------------------------------------------------------------------
// lscht das gesamte Display, indem der Speicher zeilenweise von links nach rechts
// mit 0 beschrieben wird.
void DOG_cls(void)
{
uint8_t page, col;

for (page = 0; page < MAX_PAGE; page++)
	{
	DOG_goto_xy(0, page);
	for (col = 0; col < MAX_COL; col++) DOG_write_data(0);
	}
}

/*
// ----------------------------------------------------------------------------
// berschreibt das gesamte Display, indem der Speicher zeilenweise von links nach rechts
// mit einem beliebigen Zeichen 'c' beschrieben wird.
void DOG_fill_display(char c)
{
uint8_t page, col;

for (page = 0; page < MAX_PAGE; page++)
	{
	DOG_goto_xy(0, page);
	for (col = 0; col < 21; col++) DOG_print_c(c);
	}
}
*/

// ----------------------------------------------------------------------------
// gibt genau ein Zeichen pixelweise aus,
// gedruckt wird ab der aktuellen Cursorposition des Displays
// so werden die Zeichen nacheinander weg gedruckt
//
// die globalen Variablen txt_cursor_x/y dient dazu, das Ende der Zeile zu erkennen,
// weitere Zeichen werden dann nicht mehr gedruckt
void DOG_print_c(char c)
{
char * ptr;
														/*
if (c < 32) c = 0;								// ASCII-Werte an den reduzierten Zeichensatz
else if (c > 127) c = 0;						// anpassen
else c -= 32;										*/
														// Im Zeichensatz sind die Werte fr ASCII 0 .. 31
														// auskommentiert.
														// Daher muss der Index um 32 decrementiert werden.
c -= 32;												// ohne weitere Plausibiltsprfung ausfhren
                                          // die Zeichen < ASCII 32 ignorieren

if (font_typ == FONT_NORMAL)
	{												   // abbrechen, wenn kein Platz mehr vorhanden
   if (txt_cursor_x > (MAX_COL - FONT_NORMAL_WIDTH)) return;

	ptr = &font_normal[(uint8_t)c][0];
	for(tmp = 0; tmp < FONT_NORMAL_WIDTH; tmp++) DOG_write_data(pgm_read_byte(ptr++));

	txt_cursor_x += FONT_NORMAL;				// den Textcursor mitwandern lassen
	}

else
	{													// abbrechen, wenn kein Platz mehr vorhanden
	if (txt_cursor_x > (MAX_COL - FONT_BIG_WIDTH)) return;
	ptr = &font_big[(uint8_t)c][0];

	ptr++;											// der Big-Font ist etwas unglcklich definiert
														// erst kommt die untere linke Hlfte, dann die obere
														// anschlieend die untere rechte Hlfte .......
	if (txt_cursor_y)								// wenn der Cursor in Zeile 0 steht
		{
		DOG_goto_xy(txt_cursor_x, txt_cursor_y -1);
		for(tmp = 0; tmp < FONT_BIG_WIDTH; tmp++)
			{
			DOG_write_data(pgm_read_byte(ptr++));
			ptr++;
			}
		}
	DOG_goto_xy(txt_cursor_x, txt_cursor_y);
	ptr = &font_big[(uint8_t)c][0];
	for(tmp = 0; tmp < FONT_BIG_WIDTH; tmp++)
		{
		DOG_write_data(pgm_read_byte(ptr++));
		ptr++;
		}
	txt_cursor_x += FONT_BIG_WIDTH;				// den Textcursor mitwandern lassen
	}
}

// ---------------------------------------------------------------------------
// Setzt einen Dezimalpunkt mit etwas weniger als halber Zeichenbreite
// an der aktuellen Cursorposition des Displays, sieht geflliger aus als '.'
void DOG_font_big_point(void)
{
tmp = (32 + 16 + 8);
DOG_write_data(0);
DOG_write_data(tmp);
DOG_write_data(tmp);

txt_cursor_x += 3;

}

// ---------------------------------------------------------------------------
// Setzt einen Dezimalpunkt mit etwas mehr als halber Zeichenbreite
// an der aktuellen Cursorposition des Displays, sieht geflliger aus als '.'
void DOG_font_normal_point(void)
{
tmp = (64 + 32);
DOG_write_data(0);
DOG_write_data(tmp);

txt_cursor_x += 2;
}

// ----------------------------------------------------------------------------
// gibt einen String aus
// die Ausgabe der Zeichen erfolgt ab der aktuellen Cursorposition des Displays
void DOG_print_s(char *str)
{
while (*str) DOG_print_c(*str++);
}

// ----------------------------------------------------------------------------
// gibt einen String aus dem Flash aus
// die Ausgabe der Zeichen erfolgt ab der aktuellen Cursorposition des Displays
void DOG_print_p(const char* progmem)
{
while ( (tmp = pgm_read_byte(progmem++)) ) DOG_print_c(tmp);
}

// ----------------------------------------------------------------------------
// setzt eine globale Variable fr den zu verwendenden Fonttyp
void DOG_set_font(uint8_t font)
{
if (font == FONT_BIG) font_typ = FONT_BIG; else font_typ = FONT_NORMAL;
}


// ----------------------------------------------------------------------------
// Zeichnet einen horizontalen Bargraph, am linken Display-Rand beginnend
// Parameter:
// - Zeile (Page) [0 ... 7] (von oben nach unten zhlend),
// - Wert [0 ... 100]
#ifdef H_BARGRAPH
void DOG_h_bargraph(uint8_t row, uint8_t value)
{
uint8_t i = 0;
char cv[4];											// fr utoa()
DOG_goto_xy(0, row);								// an den Zeilenanfang gehen

if (value > 100) value = 100;             // Werte ber 100 kappen

while (i++ < value) DOG_write_data(127);  // Graphik fr "vollen" Balken
while (i++ <= 100)  DOG_write_data(32);   // Graphik fr "leeren" Balken
while (i++ <  106)  DOG_write_data(0); 	// bis zum Textbeginn lschen

DOG_txt_pos(17, row);							// Textstartpunkt setzen (4 Zeichen folgen)
if (value < 10)  DOG_print_c(' '); 			// rechtsbndig drucken
if (value < 100) DOG_print_c(' ');

DOG_print_s(utoa(value, cv, 10));			// den Wert ausgeben
DOG_print_c('%');                         //

}
#endif

// --- [ eof ] ----------------------------------------------------------------
