/*----------------------------------------------------------------------------
 Copyright:
 Author:         Radig Ulrich
 Remarks:
 known Problems: none
 Version:        21.11.2009
 Description:    EA DOG (M/L)128-6
------------------------------------------------------------------------------*/
/* ---------------------------------------------------------------------------
 erstellt nach der Vorlage wie oben beschreiben

 alle Variablen des Moduls sind lokal (es werden keine globalen Variablen benutzt !),
 die Textposition wird mit DOG_txt_position() gesetzt
----------------------------------------------------------------------------- */

#ifndef _DISPLAY_H
	#define _DISPLAY_H

	#define DISPLAY_OFFSET		4
	#define MAX_COL				132
	#define MAX_PAGE				8

	//A0 Port
	#define PORT_A0  			PORTB
	#define DDR_A0   			DDRB
	#define PIN_A0   			PB0

	//Reset Port
	#define PORT_RST 			PORTB
	#define DDR_RST  			DDRB
	#define PIN_RST  			PB1

	//SPI Port
	#define SPI_PORT			PORTB
	#define SPI_DDR			DDRB

	#define SPI_SS				PB1 				//        -> Pin 40 CS1B
	#define SPI_RESET			PB2				//        -> Pin 39 RST
	#define SPI_A0				PB0				//        -> Pin 38 A0
	#define SPI_Clock			PB5 				// SCL    -> Pin 37 SCL
	#define SPI_DO				PB3 				// MOSI   -> Pin 36 SI

	#define LED_DDR			DDRD
	#define LED_PORT			PORTD
	#define LED_PIN			PD3

	#define FONT_NORMAL		0
	#define FONT_BIG			1
	#define FONT_SMALL		2

	#define H_BARGRAPH                      // nur definieren, wenn diese Funktionen benutzt wird

	void DOG_init(void);
	void DOG_lcd_contrast(int8_t);
	void DOG_cls(void);
	void DOG_fill_display(char c);			// wie DOG_cls(), jedoch wird mit 'c' gefllt
	void DOG_dimm_led(uint8_t pwm);

	void DOG_txt_pos(uint8_t spalte, uint8_t zeile);
	void DOG_goto_xy(uint8_t x, uint8_t zeile);

	void DOG_set_font(uint8_t);
   void DOG_font_big_point(void);
   void DOG_font_normal_point(void);
	void DOG_print_p(const char *);
	void DOG_print_s(char *);
	void DOG_print_c(char);
	
	void DOG_h_bargraph(uint8_t row, uint8_t value);
//	void DOG_print_bmp(char * progmem);


	//Befehlstabelle EA DOGM128-6 Seite 5
	// (1) Display ON/OFF
	#define DISPLAY_ON       				0xAF  //LCD_DISPLAY_ON
	#define DISPLAY_OFF      				0xAE  //LCD_DISPLAY_OFF

	// (2) Display start line set


	// (3) Page address set
	#define DISPLAY_PAGE_ADDRESS			0xB0

	// (4) Column address set upper bit
	#define DISPLAY_COL_ADDRESS_MSB		0x10
	// (4) Column address set lower bit
	#define DISPLAY_COL_ADDRESS_LSB		0x00

	// (5) Status read (doesn't work in SPI mode)

	// (6) Display data write

	// (7) Display data read (doesn't work in SPI mode)

	// (8) ADC select
	#define DISPLAY_BOTTOMVIEW				0xA0
	#define DISPLAY_TOPVIEW					0xA1

	// (9) Display normale/reverse
	#define DISPLAY_NORMAL   				0xA6
	#define DISPLAY_REVERSE					0xA7

	// (10) Display all points ON/OFF
	#define DISPLAY_SHOW_NORMAL			0xA4
	#define DISPLAY_SHOW_ALL_POINTS		0xA5

	// (11) LCD bias set
	#define DISPLAY_BIAS_1_9				0xA2
	#define DISPLAY_BIAS_1_7				0xA3

	// (12) Read-modify-write (doesn't work in SPI mode)

	// (13) End clear read/modify/write (doesn't work in SPI mode)

	// (14) RESET
	#define DISPLAY_RESET_CMD				0xE2

	// (15) Common output mode select
	#define DISPLAY_SCAN_DIR_NORMAL		0xC0
	#define DISPLAY_SCAN_DIR_REVERSE		0xC8

	// (16) Power control set
	#define DISPLAY_POWER_OFF				0x28
	#define DISPLAY_POWER_ALL_ON			0x2F
//	#define DISPLAY_POWER_WIDE_RANGE		0x27
//	#define DISPLAY_POWER_LOW_VOLTAGE	0x2B

	// (17) V0 voltage regulator internal resistor ratio set
	#define DISPLAY_VOLTAGE_RESISTOR   	0x26

	// (18) Electronic volume mode set
	#define DISPLAY_VOLUME_MODE_1    	0x81
	// (18) Register
	#define DISPLAY_VOLUME_MODE_2    	0x20

	// (19) Static indicator ON/OFF
	#define DISPLAY_INDICATOR_ON       	0xAD
	#define DISPLAY_INDICATOR_OFF      	0xAC
	// (19) Static indicator register set
	#define DISPLAY_INDICATOR_MODE_OFF 	0x00
	#define DISPLAY_INDICATOR_MODE_1HZ 	0x01
	#define DISPLAY_INDICATOR_MODE_2HZ 	0x10
	#define DISPLAY_INDICATOR_MODE_ON  	0x11

	// (20) Booster ratio set
	#define DISPLAY_BOOSTER_SET      	0xF8
	#define DISPLAY_BOOSTER_234      	0x00
	#define DISPLAY_BOOSTER_5        	0x01
	#define DISPLAY_BOOSTER_6        	0x03

	// (21) Power save

	// (22) NOP
	#define LCD_NOP              			0xE3

	// (23) Test Command for IC test. Do not use this command.

#endif //_DISPLAY_H





