// --- [ DOG_dia.c ] ----------------------------------------------------------
// Ist zustndig fr die Darstellung einer Folge von (Mess-)Werte in graphischer Form als:
// - Diagramm mit konstanten Werten der Y-Achse,
// - Diagramm mit floatenden Werten der Y-Achse,
//   (der Y-Wert wird an die aktuellen Messwerte angepasst)
//
// Fr die Routinen dieses Moduls ist kein Grafikspeicher erforderlich !

#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <ctype.h>
#include <avr/io.h>
#include <util/delay.h>
#include "global.h"
#include "dog128x64.h"
                                          // Grafikbereich von oben links gezhlt
#define START_ROW          2              // Start Frame
#define START_COL          8              // Start Column
#define TAB_ROWS           6					// Anzahl der Frames
#define TAB_COLS           128            // Anzahl der Columns
                                          // fr die gestrichelte Referenzlinien:
#define R_LINE_X_PIX_DIST   5             // horizontaler Abstand der Punkte der Referenzlinien
#define R_LINE_Y_PIX_DIST  10             // vertikaler Abstand der Referenzlinien


#if DEMO == DIAGRAMM_FLOAT_DEMO
#define DIA_FLOAT
#endif
#if DEMO == DIAGRAMM_FIXED_DEMO
#define DIA_FIXED									// auskommentieren, wenn diese Funktionen nicht benutzt werden
#endif

extern void DOG_write_data(uint8_t);

void set_point_sram(uint8_t data, uint8_t * scr);
//void make_dez_point(void);

extern char msg[];
// --- lokale Variablen

uint32_t mittelwert = 0;
uint16_t messwerte[TAB_COLS];
uint8_t display_row[START_ROW + TAB_ROWS];

// ----------------------------------------------------------------------------
// nur fr Testzwecke
// fllt das Array der Messwerte beim Programmstart vorab mit einem festem Wert
void fill_array(uint16_t data)
{
uint8_t i;
for (i =  0; i < TAB_COLS; i++) messwerte[i] = data;
}


// ----------------------------------------------------------------------------
// Zeichnet ein Diagramm, dessen Y-Achse konstante Y-Werte darstellt.
// Wenn der Messwert den darstellbaren Y-Bereich verlsst, dann ist der Graph
// nicht mehr sichtbar !
// Gedacht zur Darstellung/berwachung von Werten innerhalb eines festen Wertebereiches.
//
// Zeigt in den beiden oberen Rows den aktuellen Messwert als BIG-Font
// sowie den Maximal- und den Minimalwert innerhalb der dargestellten Datenreihe.
//
// Fgt einen neuen Messwert ins Array ein, incrementiert den Index auf die nchste
// Speicherstelle und baut anschlieend das Diagramm neu auf.
// Der aktuelle Messwert wird grafisch an linken Displayrand ausgegeben, nach rechts
// folgen die jeweils lteren Messwerte.
// Der aktuelle Messwert wird numerisch im Big-Font auf dem Display ausgegeben
//
#ifdef DIA_FIXED
void DOG_dia_fixed(uint16_t data)
{
#define Y_OFFSET_R_LINE    3              // Y-Wert der ersten gepunkteten Referenz-Linie
#define Y_START            477            // Messwert fr y = 0 im Diagramm
// es ist der Wertebereich von Y_START bis Y_START + (TAB_ROWS * 8 - 1) darstellbar
// bei Y_START 480 und 6 Zeilen also von 480 bis 527


static uint8_t top = 0;                   // Index auf den nchsten Speicherplatz fr Messwerte
uint8_t i, j, idx, r, y_value;
uint16_t max = 0, min = 0xFFFF;

idx = top;										   // zustzlicher Index auf die Messwerte
messwerte[top++] = data;						// den Messwert ins Array einreihen
if (top >= TAB_COLS) top = 0;             // Index-berlauf am Ende des Arrays

for (i = 0; i < (TAB_COLS); i++)				// die Extreme aller gespeicherter Messungen bestimmen
	{
	if (messwerte[i] < min) min = messwerte[i];
	if (messwerte[i] > max) max = messwerte[i];
	}

// der Messwert sei eine Spannung gemessen in der Einheit 10mV
// jedes Pixel in Y-Richtung stellt 10 mV dar.


r = 0;                                    // falls die Tabelle nicht in Col = 0 beginnt,
                                          // separater Zhler fr die Spalten der Tabelle

for (i = START_COL; i < (START_COL + TAB_COLS); i++) // Schleife fr die X-Werte
   {
                                          // den Display-Buffer lschen
  for (j = START_ROW; j < (START_ROW +TAB_ROWS);  j++) display_row[j] = 0;

                                          // zuerst die gepunktete Referenzlinie vorbereiten
	                                       // r sorgt dafr, dass der 1. Punkt immer am linken Tabellenrand steht.
   if (!(r++ % R_LINE_X_PIX_DIST))
      {
      for (j = Y_OFFSET_R_LINE; j < (TAB_ROWS * 8); j += R_LINE_Y_PIX_DIST)
         {
         set_point_sram(j, display_row);
         }
      }
	                                       // den Messwert auf die Displaykoordinaten bringen
   y_value = (uint8_t) (messwerte[idx] - Y_START);
	                                      // der Index luft rckwrts - entgegen der Zeitachse !
	if (idx) idx--; else idx = (TAB_COLS - 1);// Index berlauf

                                          // in der Tabelle nicht darstellbare Werte ausblenden
   if ((y_value < (TAB_ROWS * 8)))
//  if ((y_value < (Y_START + TAB_ROWS * 8)) && (y_value >= Y_START))
		{
		set_point_sram(y_value, display_row); // den Messwert in den Speicher schreiben
		}
                                          // alle Pages dieser Spalte schreiben
   for (j = START_ROW; j < (START_ROW + TAB_ROWS); j++)
      {
		DOG_goto_xy(i, j);						// Cursor positionieren
		DOG_write_data(display_row[j]);		// Daten schreiben
      }
	}
	
// --- Text ausgeben
DOG_set_font(FONT_BIG);
DOG_txt_pos(0, 1);
DOG_print_s(utoa((data), msg,10));			// Messwert
DOG_print_s("0mV");								// hier wird noch ein 0 drangehngt !!!

DOG_set_font(FONT_NORMAL);

DOG_goto_xy(80, 0);
DOG_print_p(PSTR("max:"));						// grter Anzeigewert
DOG_print_s(utoa(max, msg,10));
DOG_print_c('0');

DOG_goto_xy(80, 1);
DOG_print_p(PSTR("min:"));						// kleinster Anzeigewert
DOG_print_s(utoa(min, msg,10));
DOG_print_c('0');
}

#endif


// ----------------------------------------------------------------------------
// Zeichnet ein Diagramm, dessen Y-Achse den Y-Messwerten folgt,
// der Graph bleibt auch bei greren Messwertnderungen (gilt zumindest die neueren Werte)
// im sichtbaren Bereich.
// Gedacht zur Darstellung der Luftdruckvernderungen eines Barometers - also fr langsame
// und nicht sprunghafte Messwertnderungen.
//
// Zeigt in den beiden oberen Rows den aktuellen Messwert als BIG-Font
// sowie den Maximal- und den Minimalwert innerhalb der dargestellten Datenreihe.
//
// Fgt einen neuen Messwert ins Array ein, incrementiert den Pointer auf das nchste Element
// Der aktuelle Messwert wird numerisch in groen Zeichen auf dem Display ausgegeben,
// der Minimal- und der Maximalwert aus den gespeicherten Daten jeweils in kleiner Schrift.
//
#ifdef DIA_FLOAT
void DOG_dia_float(uint16_t data)
{
static uint8_t top = 0;                   // Index auf den nchsten Speicherplatz fr Messwerte
uint8_t i, j, idx, r, z, y_value;
uint16_t y0_value, max, min;              //

idx = top;											// lokaler Index auf die Messwerte
messwerte[top++] = data;						// den Messwert ins Array einreihen
if (top >= TAB_COLS) top = 0;             // Index-berlauf am Ende des Arrays

max = 0;												// Vorgaben fr min und max
min = 0xFFFF;

/* ---- den Startpunkt der Grafik aus dem Mittelwert aller Messungen bestimmen
mittelwert = 0;
for (i = 0; i < (TAB_COLS); i++)				// den Mittelwert aller Messungen bestimmen
	{
	mittelwert += messwerte[i];
	if (messwerte[i] < min) min = messwerte[i];
	if (messwerte[i] > max) max = messwerte[i];
	}
mittelwert /= (TAB_COLS);
*/

// ---- den Startpunkt der Grafik aus dem Mittelwert der letzten i-Messungen bestimmen
//*

#define MESSUNGEN	64								// Anzahl der zu bercksichtigen Messwert				
i = MESSUNGEN;
j = idx;												// Index des letzten Messwertes
mittelwert = 0;									// Mittelwert zurcksetzen
while(i--)											// i-Messwerte aufsummieren
	{
	mittelwert += messwerte[j--];				// den Index decrementieren
	if (j == 255) j = (TAB_COLS - 1);		// falls berlauf von 0 auf 255
	}
mittelwert /= MESSUNGEN;

for (i = 0; i < (TAB_COLS); i++)				// die Extremwerte der Messungen bestimmen
	{
	if (messwerte[i] < min) min = messwerte[i];
	if (messwerte[i] > max) max = messwerte[i];
	}
//*/

// der Messwert ist der mit 10 multiplizierte Luftdruck in mBar
// jeder Pixel in Y-Richtung stellt 1/10 mBar dar.

// den Wert fr die Anzeigenmitte berechnen:
// dazu vom Durchschnittwert die halbe Windowshhe subtrahieren
// dann sollte der Graph in der Bildschirmmitte starten.
y0_value = (uint16_t) mittelwert - ((TAB_ROWS * 8) / 2 );

// falls der Startpunkt aber aus dem Sichtfeld wandert, weil der aktuelle
// Wert und der Mittelwert zu weit auseinander driften ...
if (data >= (y0_value + (TAB_ROWS * 8 - 2))) y0_value = data - (TAB_ROWS * 8 - 2);
else if (data < (y0_value + 2)) y0_value = data - 2;
														// Alternativ:
// y0_value = data - (TAB_ROWS * 8 / 2);  // Der Graph startet immer genau in der Mitte

// Zum Abschtzen der Messwerte werden in 10er Schritten Referenzlinien als
// gepunktete Linien im Diagramm ausgegeben.
// Den Y-Wert fr die 1. Referenzlinie (die letzte Ziffer ist 0) finden
z = 10 - (y0_value % 10);					   // die letzte Ziffer abschneiden
if (z == 10) z = 0;							   //

r = 0;                                    // falls die Tabelle nicht in Col = 0 beginnt
                                          // separater Spaltenzhler fr die Tabelle

for (i = START_COL; i < (START_COL + TAB_COLS); i++) // Schleife fr die X-Werte
   {
                                          // den Display-Buffer lschen
  for (j = START_ROW; j < (START_ROW +TAB_ROWS);  j++) display_row[j] = 0;

                                          // zuerst die gepunktete Referenzlinie vorbereiten
	                                       // r sorgt dafr, dass der 1. Punkt immer am linken Tabellenrand steht.
   if (!(r++ % R_LINE_X_PIX_DIST))
      {
      for (j = z; j < (TAB_ROWS * 8); j += R_LINE_Y_PIX_DIST)
         {
         set_point_sram(j, display_row);
         }
      }
	                                       // den Messwert auf die Displaykoordinaten bringen
   y_value = (uint8_t) (messwerte[idx] - y0_value);
	                                      // der Index luft rckwrts - entgegen der Zeitachse !
	if (idx) idx--; else idx = (TAB_COLS - 1);// Index berlauf

                                          // in der Tabelle nicht darstellbare Werte ausblenden
                                          // negative Werte werden hier indirekt bercksichtigt,
                                          // wegen berlaufs  von 0 auf 0xFF
   if (y_value < TAB_ROWS * 8)
		{
		set_point_sram(y_value, display_row); // den Messwert in den Speicher schreiben
		}
                                          // alle Pages dieser Spalte schreiben
   for (j = START_ROW; j < (START_ROW + TAB_ROWS); j++)
      {
		DOG_goto_xy(i, j);						// Cursor positionieren
		DOG_write_data(display_row[j]);		// Daten schreiben
      }
	}

// --- Text ausgeben
	DOG_set_font(FONT_BIG);
	DOG_txt_pos(0, 1);
	DOG_print_s(utoa(data/10, msg,10));
	DOG_font_big_point();
	DOG_print_s(utoa(data % 10, msg,10));

	DOG_set_font(FONT_NORMAL);
	DOG_txt_pos(12,0);
	DOG_print_p(PSTR("max:"));

	DOG_print_s(utoa(max / 10, msg,10));
	DOG_font_normal_point();
	DOG_print_s(utoa(max % 10, msg,10));

	DOG_txt_pos(12,1);
	DOG_print_p(PSTR("min:"));
 	
	DOG_print_s(utoa(min / 10, msg,10));
	DOG_font_normal_point();
	DOG_print_s(utoa(min % 10, msg,10));

}
#endif
/*
// ----------------------------------------------------------------------------
// fgt einen Dezimalpunkt vor dem letzten Zeichen des String msg[] ein
void make_dez_point(void)
{
uint8_t i = 0;
while (msg[i]) i++;
msg[i] = msg[i-1];
msg[i - 1] = '.';
msg[i + 1] = 0;
}
*/

// ----------------------------------------------------------------------------
// setzt einen Punkt in einem Array, das den gesamten "Grafikbereich" einer
// zu bearbeitenden Spalte (!) reprsentiert,
// der Grafikbereich ist also maximal 8 Byte gro.
// Der Y-Wert wird transformiert auf die Organisation des Displays:
// - Ursprung oben links,
// - Y-Achse nach unten zeigend

void set_point_sram(uint8_t data, uint8_t * scr)
{
const uint8_t pix_mask[8] = {1, 2, 4 ,8, 16, 32, 64, 128};

// den Y-Wert umrechnen, Eingabe mit Ursprung links unten
// Eingabe 0 = unten in der Tabelle
// Display 0 = oberen im Display

data  = ((TAB_ROWS * 8) -1) - data;
data += 8 * START_ROW;
scr[data / 8] |= pix_mask[data % 8];
}



// --- [ eof ] ----------------------------------------------------------------
